# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base

CONF = config.CONF


class NetworksRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def resource_setup(cls):
        super(NetworksRbacTest, cls).resource_setup()
        cls.network = cls.create_network()
        cls.cidr = netaddr.IPNetwork(CONF.network.project_network_cidr)
        cls.subnet = cls.create_subnet(
            cls.network, cidr=cls.cidr, mask_bits=24, enable_dhcp=False)

    def _create_network(self,
                        router_external=None,
                        router_private=None,
                        provider_network_type=None,
                        provider_physical_network=None,
                        provider_segmentation_id=None,
                        **kwargs):
        if router_external is not None:
            kwargs['router:external'] = router_external
        if router_private is not None:
            kwargs['router:private'] = router_private
        if provider_network_type is not None:
            kwargs['provider:network_type'] = provider_network_type
        if provider_physical_network is not None:
            kwargs['provider:physical_network'] = provider_physical_network
        if provider_segmentation_id is not None:
            kwargs['provider:segmentation_id'] = provider_segmentation_id

        network_name = data_utils.rand_name(
            self.__class__.__name__ + '-Network')
        network = self.create_network(network_name=network_name, **kwargs)
        return network

    def _update_network(self,
                        net_id=None,
                        admin=None,
                        shared_network=None,
                        router_external=None,
                        router_private=None,
                        segments=None,
                        **kwargs):
        if not net_id:
            net_id = self.network['id']

        if admin is not None:
            kwargs['admin_state_up'] = admin
        elif shared_network is not None:
            kwargs['shared'] = shared_network
        elif router_external is not None:
            kwargs['router:external'] = router_external
        elif router_private is not None:
            kwargs['router:private'] = router_private
        elif segments is not None:
            kwargs['segments'] = segments

        updated_network = self.networks_client.update_network(
            net_id, **kwargs)['network']
        return updated_network

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_network")
    @decorators.idempotent_id('95b9baab-1ece-4e2b-89c8-8d671d974e54')
    def test_create_network(self):

        """Create Network Test

        RBAC test for the neutron create_network policy
        """
        with self.rbac_utils.override_role(self):
            self._create_network()

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_network:shared")
    @decorators.idempotent_id('ccabf2a9-28c8-44b2-80e6-ffd65d43eef2')
    def test_create_network_shared(self):

        """Create Shared Network Test

        RBAC test for the neutron create_network:shared policy
        """
        with self.rbac_utils.override_role(self):
            self._create_network(shared=True)

    @utils.requires_ext(extension='external-net', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_network:router:external")
    @decorators.idempotent_id('51adf2a7-739c-41e0-8857-3b4c460cbd24')
    def test_create_network_router_external(self):

        """Create External Router Network Test

        RBAC test for the neutron create_network:router:external policy
        """
        with self.rbac_utils.override_role(self):
            self._create_network(router_external=True)

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_network:provider:network_type")
    @decorators.idempotent_id('3c42f7b8-b80c-44ef-8fa4-69ec4b1836bc')
    def test_create_network_provider_network_type(self):

        """Create Provider Network Test

        RBAC test for the neutron create_network:provider:network_type policy
        """
        with self.rbac_utils.override_role(self):
            self._create_network(provider_network_type='vxlan')

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rule="create_network:provider:segmentation_id")
    @decorators.idempotent_id('b9decb7b-68ef-4504-b99b-41edbf7d2af5')
    def test_create_network_provider_segmentation_id(self):

        """Create Provider Network Segmentation Id Test

        RBAC test for the neutron create_network:provider:segmentation_id
        """
        with self.rbac_utils.override_role(self):
            self._create_network(provider_network_type='vxlan',
                                 provider_segmentation_id=200)

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_network")
    @decorators.idempotent_id('6485bb4e-e110-48ae-83e1-3ec8b40c3107')
    def test_update_network(self):

        """Update Network Test

        RBAC test for the neutron update_network policy
        """
        updated_name = data_utils.rand_name(
            self.__class__.__name__ + '-Network')

        with self.rbac_utils.override_role(self):
            self._update_network(name=updated_name)

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_network:shared")
    @decorators.idempotent_id('37ea3e33-47d9-49fc-9bba-1af98fbd46d6')
    def test_update_network_shared(self):

        """Update Shared Network Test

        RBAC test for the neutron update_network:shared policy
        """
        with self.rbac_utils.override_role(self):
            self._update_network(shared_network=True)
        self.addCleanup(self._update_network, shared_network=False)

    @utils.requires_ext(extension='external-net', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="update_network:router:external")
    @decorators.idempotent_id('34884c22-499b-4960-97f1-e2ed8522a9c9')
    def test_update_network_router_external(self):

        """Update Network Router External Test

        RBAC test for the neutron update_network:router:external policy
        """
        network = self._create_network()
        with self.rbac_utils.override_role(self):
            self._update_network(net_id=network['id'], router_external=True)

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_network")
    @decorators.idempotent_id('0eb62d04-338a-4ff4-a8fa-534e52110534')
    def test_show_network(self):

        """Show Network Test

        RBAC test for the neutron get_network policy
        """
        with self.rbac_utils.override_role(self):
            self.networks_client.show_network(self.network['id'])

    @utils.requires_ext(extension='external-net', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_network:router:external")
    @decorators.idempotent_id('529e4814-22e9-413f-af48-8fefcd637344')
    def test_show_network_router_external(self):

        """Show Network Router External Test

        RBAC test for the neutron get_network:router:external policy
        """
        kwargs = {'fields': 'router:external'}

        with self.rbac_utils.override_role(self):
            self.networks_client.show_network(self.network['id'],
                                              **kwargs)

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_network:provider:network_type")
    @decorators.idempotent_id('6521dd60-0950-458b-8491-09d3c84ac0f4')
    def test_show_network_provider_network_type(self):

        """Show Network Prodiver Network Type Test

        RBAC test for the neutron get_network:provider:network_type policy
        """
        kwargs = {'fields': 'provider:network_type'}

        with self.rbac_utils.override_role(self):
            retrieved_network = self.networks_client.show_network(
                self.network['id'], **kwargs)['network']

        if len(retrieved_network) == 0:
            raise rbac_exceptions.RbacMalformedResponse(True)

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_network:provider:physical_network")
    @decorators.idempotent_id('c049f11a-240c-4a85-ad43-a4d3fd0a5e39')
    def test_show_network_provider_physical_network(self):

        """Show Network Provider Physical Network Test

        RBAC test for the neutron get_network:provider:physical_network policy
        """
        kwargs = {'fields': 'provider:physical_network'}

        with self.rbac_utils.override_role(self):
            retrieved_network = self.networks_client.show_network(
                self.network['id'], **kwargs)['network']

        if len(retrieved_network) == 0:
            raise rbac_exceptions.RbacMalformedResponse(empty=True)

    @utils.requires_ext(extension='provider', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_network:provider:segmentation_id")
    @decorators.idempotent_id('38d9f085-6365-4f81-bac9-c53c294d727e')
    def test_show_network_provider_segmentation_id(self):

        """Show Network Provider Segmentation Id Test

        RBAC test for the neutron get_network:provider:segmentation_id policy
        """
        kwargs = {'fields': 'provider:segmentation_id'}

        with self.rbac_utils.override_role(self):
            retrieved_network = self.networks_client.show_network(
                self.network['id'], **kwargs)['network']

        if len(retrieved_network) == 0:
            raise rbac_exceptions.RbacMalformedResponse(empty=True)

        key = retrieved_network.get('provider:segmentation_id', "NotFound")
        self.assertNotEqual(key, "NotFound")

    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_network")
    @decorators.idempotent_id('56ca50ed-ac58-49d6-b239-ed39e7124d5c')
    def test_delete_network(self):

        """Delete Network Test

        RBAC test for the neutron delete_network policy
        """
        network = self._create_network()
        with self.rbac_utils.override_role(self):
            self.networks_client.delete_network(network['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_subnet")
    @decorators.idempotent_id('44f42aaf-8a9a-4678-868a-b8fe82689554')
    def test_create_subnet(self):

        """Create Subnet Test

        RBAC test for the neutron create_subnet policy
        """
        network = self._create_network()

        with self.rbac_utils.override_role(self):
            self.create_subnet(network, enable_dhcp=False)

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_subnet")
    @decorators.idempotent_id('eb88be84-2465-482b-a40b-5201acb41152')
    def test_show_subnet(self):

        """Show Subnet Test

        RBAC test for the neutron get_subnet policy
        """
        with self.rbac_utils.override_role(self):
            self.subnets_client.show_subnet(self.subnet['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_subnet")
    @decorators.idempotent_id('1bfeaec5-83b9-4140-8138-93a0a9d04cee')
    def test_update_subnet(self):

        """Update Subnet Test

        RBAC test for the neutron update_subnet policy
        """
        updated_name = data_utils.rand_name(
            self.__class__.__name__ + '-Network')

        with self.rbac_utils.override_role(self):
            self.subnets_client.update_subnet(self.subnet['id'],
                                              name=updated_name)

    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_subnet")
    @decorators.idempotent_id('1ad1400f-dc84-4edb-9674-b33bbfb0d3e3')
    def test_delete_subnet(self):

        """Delete Subnet Test

        RBAC test for the neutron delete_subnet policy
        """
        network = self._create_network()
        subnet = self.create_subnet(network, enable_dhcp=False)

        with self.rbac_utils.override_role(self):
            self.subnets_client.delete_subnet(subnet['id'])

    @utils.requires_ext(extension='dhcp_agent_scheduler', service='network')
    @decorators.idempotent_id('b524f19f-fbb4-4d11-a85d-03bfae17bf0e')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_dhcp-agents")
    def test_list_dhcp_agents_on_hosting_network(self):

        """List DHCP Agents on Hosting Network Test

        RBAC test for the neutron "get_dhcp-agents" policy
        """
        with self.rbac_utils.override_role(self):
            self.networks_client.list_dhcp_agents_on_hosting_network(
                self.network['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentitySericesV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_service")
    @decorators.idempotent_id('9a4bb317-f0bb-4005-8df0-4b672885b7c8')
    def test_create_service(self):
        with self.rbac_utils.override_role(self):
            self.setup_test_service()

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_service")
    @decorators.idempotent_id('b39447d1-2cf6-40e5-a899-46f287f2ecf0')
    def test_update_service(self):
        service = self.setup_test_service()
        new_name = data_utils.rand_name(self.__class__.__name__ + '-service')

        with self.rbac_utils.override_role(self):
            self.services_client.update_service(service['id'],
                                                service=service,
                                                name=new_name,
                                                type=service['type'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_service")
    @decorators.idempotent_id('177b991a-438d-4bef-8e9f-9c6cc5a1c9e8')
    def test_delete_service(self):
        service = self.setup_test_service()

        with self.rbac_utils.override_role(self):
            self.services_client.delete_service(service['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_service")
    @decorators.idempotent_id('d89a9ac6-cd53-428d-84c0-5bc71f4a432d')
    def test_show_service(self):
        service = self.setup_test_service()

        with self.rbac_utils.override_role(self):
            self.services_client.show_service(service['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_services")
    @decorators.idempotent_id('706e6bea-3385-4718-919c-0b5121395806')
    def test_list_services(self):
        with self.rbac_utils.override_role(self):
            self.services_client.list_services()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class MeteringLabelsRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(MeteringLabelsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('metering', 'network'):
            msg = "metering extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(MeteringLabelsRbacTest, cls).setup_clients()
        cls.metering_labels_client = cls.os_primary.metering_labels_client

    def _create_metering_label(self):
        body = self.metering_labels_client.create_metering_label(
            name=data_utils.rand_name(self.__class__.__name__))

        label = body['metering_label']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.metering_labels_client.delete_metering_label,
                        label['id'])
        return label

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_metering_label")
    @decorators.idempotent_id('e8cfc8b8-c159-48f0-93b3-591625a02f8b')
    def test_create_metering_label(self):
        """Create metering label.

        RBAC test for the neutron "create_metering_label" policy
        """
        with self.rbac_utils.override_role(self):
            self._create_metering_label()

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_metering_label",
                                 expected_error_code=404)
    @decorators.idempotent_id('c57f6636-c702-4755-8eac-5e73bc1f7d14')
    def test_show_metering_label(self):
        """Show metering label.

        RBAC test for the neutron "get_metering_label" policy
        """
        label = self._create_metering_label()
        with self.rbac_utils.override_role(self):
            self.metering_labels_client.show_metering_label(label['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_metering_label",
                                 expected_error_code=404)
    @decorators.idempotent_id('1621ccfe-2e3f-4d16-98aa-b620f9d00404')
    def test_delete_metering_label(self):
        """Delete metering label.

        RBAC test for the neutron "delete_metering_label" policy
        """
        label = self._create_metering_label()
        with self.rbac_utils.override_role(self):
            self.metering_labels_client.delete_metering_label(label['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import json
import os

from oslo_log import log as logging
from oslo_policy import policy
import stevedore
from tempest import clients
from tempest.common import credentials_factory as credentials
from tempest import config

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin.rbac_utils import RbacAuthority

CONF = config.CONF
LOG = logging.getLogger(__name__)


class PolicyAuthority(RbacAuthority):
    """A class that uses ``oslo.policy`` for validating RBAC."""

    def __init__(self, project_id, user_id, service, extra_target_data=None):
        """Initialization of Policy Authority class.

        Validates whether a test role can perform a policy action by querying
        ``oslo.policy`` with necessary test data.

        If a policy file does not exist, checks whether the policy file is
        registered as a namespace under "oslo.policy.policies". Nova, for
        example, doesn't use a policy file by default; its policies are
        implemented in code and registered as "nova" under
        "oslo.policy.policies".

        If the policy file is not found in either code or in a policy file,
        then an exception is raised.

        Additionally, if a custom policy file exists along with the default
        policy in code implementation, the custom policy is prioritized.

        :param uuid project_id: project_id of object performing API call
        :param uuid user_id: user_id of object performing API call
        :param string service: service of the policy file
        :param dict extra_target_data: dictionary containing additional object
            data needed by oslo.policy to validate generic checks

        Example:

        .. code-block:: python

            # Below is the default policy implementation in code, defined in
            # a service like Nova.
            test_policies = [
                policy.DocumentedRuleDefault(
                    'service:test_rule',
                    base.RULE_ADMIN_OR_OWNER,
                    "This is a description for a test policy",
                    [
                        {
                            'method': 'POST',
                            'path': '/path/to/test/resource'
                        }
                    ]),
                    'service:another_test_rule',
                    base.RULE_ADMIN_OR_OWNER,
                    "This is a description for another test policy",
                    [
                        {
                            'method': 'GET',
                            'path': '/path/to/test/resource'
                        }
                    ]),
            ]

        .. code-block:: yaml

            # Below is the custom override of the default policy in a YAML
            # policy file. Note that the default rule is "rule:admin_or_owner"
            # and the custom rule is "rule:admin_api". The `PolicyAuthority`
            # class will use the "rule:admin_api" definition for this policy
            # action.
            "service:test_rule" : "rule:admin_api"

            # Note below that no override is provided for
            # "service:another_test_rule", which means that the default policy
            # rule is used: "rule:admin_or_owner".
        """

        if extra_target_data is None:
            extra_target_data = {}

        self.validate_service(service)

        # Prioritize dynamically searching for policy files over relying on
        # deprecated service-specific policy file locations.
        self.path = None
        if CONF.patrole.custom_policy_files:
            self.discover_policy_files()
            self.path = self.policy_files.get(service)

        self.rules = policy.Rules.load(self._get_policy_data(service),
                                       'default')
        self.project_id = project_id
        self.user_id = user_id
        self.extra_target_data = extra_target_data

    @classmethod
    def validate_service(cls, service):
        """Validate whether the service passed to ``__init__`` exists."""
        service = service.lower().strip() if service else None

        # Cache the list of available services in memory to avoid needlessly
        # doing an API call every time.
        if not hasattr(cls, 'available_services'):
            admin_mgr = clients.Manager(
                credentials.get_configured_admin_credentials())
            services_client = (admin_mgr.identity_services_v3_client
                               if CONF.identity_feature_enabled.api_v3
                               else admin_mgr.identity_services_client)
            services = services_client.list_services()['services']
            cls.available_services = [s['name'] for s in services]

        if not service or service not in cls.available_services:
            LOG.debug("%s is NOT a valid service.", service)
            raise rbac_exceptions.RbacInvalidService(
                "%s is NOT a valid service." % service)

    @classmethod
    def discover_policy_files(cls):
        """Dynamically discover the policy file for each service in
        ``cls.available_services``. Pick the first candidate path found
        out of the potential paths in ``[patrole] custom_policy_files``.
        """
        if not hasattr(cls, 'policy_files'):
            cls.policy_files = {}
            for service in cls.available_services:
                for candidate_path in CONF.patrole.custom_policy_files:
                    if os.path.isfile(candidate_path % service):
                        cls.policy_files.setdefault(service,
                                                    candidate_path % service)

    def allowed(self, rule_name, role):
        """Checks if a given rule in a policy is allowed with given role.

        :param string rule_name: Rule to be checked using ``oslo.policy``.
        :param bool is_admin: Whether admin context is used.
        """
        is_admin_context = self._is_admin_context(role)
        is_allowed = self._allowed(
            access=self._get_access_token(role),
            apply_rule=rule_name,
            is_admin=is_admin_context)
        return is_allowed

    def _get_policy_data(self, service):
        file_policy_data = {}
        mgr_policy_data = {}
        policy_data = {}

        # Check whether policy file exists and attempt to read it.
        if self.path and os.path.isfile(self.path):
            try:
                with open(self.path, 'r') as policy_file:
                    file_policy_data = policy_file.read()
                file_policy_data = json.loads(file_policy_data)
            except (IOError, ValueError) as e:
                msg = "Failed to read policy file for service. "
                if isinstance(e, IOError):
                    msg += "Please check that policy path exists."
                else:
                    msg += "JSON may be improperly formatted."
                LOG.debug(msg)
                file_policy_data = {}

        # Check whether policy actions are defined in code. Nova and Keystone,
        # for example, define their default policy actions in code.
        mgr = stevedore.named.NamedExtensionManager(
            'oslo.policy.policies',
            names=[service],
            on_load_failure_callback=None,
            invoke_on_load=True,
            warn_on_missing_entrypoint=False)

        if mgr:
            policy_generator = {policy.name: policy.obj for policy in mgr}
            if policy_generator and service in policy_generator:
                for rule in policy_generator[service]:
                    mgr_policy_data[rule.name] = str(rule.check)

        # If data from both file and code exist, combine both together.
        if file_policy_data and mgr_policy_data:
            # Add the policy actions from code first.
            for action, rule in mgr_policy_data.items():
                policy_data[action] = rule
            # Overwrite with any custom policy actions defined in policy.json.
            for action, rule in file_policy_data.items():
                policy_data[action] = rule
        elif file_policy_data:
            policy_data = file_policy_data
        elif mgr_policy_data:
            policy_data = mgr_policy_data
        else:
            error_message = (
                'Policy file for {0} service neither found in code nor at {1}.'
                .format(service, [loc % service for loc in
                                  CONF.patrole.custom_policy_files])
            )
            raise rbac_exceptions.RbacParsingException(error_message)

        try:
            policy_data = json.dumps(policy_data)
        except (TypeError, ValueError):
            error_message = 'Policy file for {0} service is invalid.'.format(
                service)
            raise rbac_exceptions.RbacParsingException(error_message)

        return policy_data

    def _is_admin_context(self, role):
        """Checks whether a role has admin context.

        If context_is_admin is contained in the policy file, then checks
        whether the given role is contained in context_is_admin. If it is not
        in the policy file, then default to context_is_admin: admin.
        """
        if 'context_is_admin' in self.rules.keys():
            return self._allowed(
                access=self._get_access_token(role),
                apply_rule='context_is_admin')
        return role == CONF.identity.admin_role

    def _get_access_token(self, role):
        access_token = {
            "token": {
                "roles": [
                    {
                        "name": role
                    }
                ],
                "project_id": self.project_id,
                "tenant_id": self.project_id,
                "user_id": self.user_id
            }
        }
        return access_token

    def _allowed(self, access, apply_rule, is_admin=False):
        """Checks if a given rule in a policy is allowed with given ``access``.

        :param dict access: Dictionary from ``_get_access_token``.
        :param string apply_rule: Rule to be checked using ``oslo.policy``.
        :param bool is_admin: Whether admin context is used.
        """
        access_data = copy.copy(access['token'])
        access_data['roles'] = [role['name'] for role in access_data['roles']]
        access_data['is_admin'] = is_admin
        # TODO(felipemonteiro): Dynamically calculate is_admin_project rather
        # than hard-coding it to True. is_admin_project cannot be determined
        # from the role, but rather from project and domain names. See
        # _populate_is_admin_project in keystone.token.providers.common
        # for more information.
        access_data['is_admin_project'] = True

        class Object(object):
            pass
        o = Object()
        o.rules = self.rules

        target = {"project_id": access_data['project_id'],
                  "tenant_id": access_data['project_id'],
                  "network:tenant_id": access_data['project_id'],
                  "user_id": access_data['user_id']}
        if self.extra_target_data:
            target.update(self.extra_target_data)

        result = self._try_rule(apply_rule, target, access_data, o)
        return result

    def _try_rule(self, apply_rule, target, access_data, o):
        if apply_rule not in self.rules:
            message = ("Policy action \"{0}\" not found in policy file: {1} or"
                       " among registered policy in code defaults for service."
                       ).format(apply_rule, self.path)
            LOG.debug(message)
            raise rbac_exceptions.RbacParsingException(message)
        else:
            rule = self.rules[apply_rule]
            return rule(target, access_data, o)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class VolumeTypesAccessRbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(VolumeTypesAccessRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-volume-type-access', 'volume'):
            msg = "os-volume-type-access extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(VolumeTypesAccessRbacTest, cls).resource_setup()
        cls.vol_type = cls.create_volume_type(
            **{'os-volume-type-access:is_public': False})
        cls.project_id = cls.os_primary.credentials.project_id

    def _add_type_access(self, ignore_not_found=False):
        self.volume_types_client.add_type_access(
            self.vol_type['id'], project=self.project_id)

        if ignore_not_found:
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.volume_types_client.remove_type_access,
                            self.vol_type['id'], project=self.project_id)
        else:
            self.addCleanup(self.volume_types_client.remove_type_access,
                            self.vol_type['id'], project=self.project_id)

    @decorators.idempotent_id('af70e6ad-e931-419f-9200-8bcc284e4e47')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_type_access")
    def test_list_type_access(self):
        self._add_type_access()

        with self.rbac_utils.override_role(self):
            self.volume_types_client.list_type_access(self.vol_type['id'])[
                'volume_type_access']

    @decorators.idempotent_id('b462eeba-45d0-4d6e-945a-a1d27708d367')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_type_access:addProjectAccess")
    def test_add_type_access(self):
        with self.rbac_utils.override_role(self):
            self._add_type_access(ignore_not_found=True)

    @decorators.idempotent_id('8f848aeb-636a-46f1-aeeb-e2a60e9d2bfe')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_type_access:removeProjectAccess")
    def test_remove_type_access(self):
        self._add_type_access(ignore_not_found=True)

        with self.rbac_utils.override_role(self):
            self.volume_types_client.remove_type_access(
                self.vol_type['id'], project=self.project_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

QUOTA_KEYS = ['gigabytes', 'snapshots', 'volumes', 'backups',
              'backup_gigabytes', 'per_volume_gigabytes']


class VolumeQuotasV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def setup_credentials(cls):
        super(VolumeQuotasV3RbacTest, cls).setup_credentials()
        cls.demo_tenant_id = cls.os_primary.credentials.tenant_id

    @classmethod
    def setup_clients(cls):
        super(VolumeQuotasV3RbacTest, cls).setup_clients()
        cls.quotas_client = cls.os_primary.volume_quotas_v2_client

    def _restore_default_quota_set(self):
        default_quota_set = self.quotas_client.show_default_quota_set(
            self.demo_tenant_id)['quota_set']
        cleanup_quota_set = dict(
            (k, v) for k, v in default_quota_set.items()
            if k in QUOTA_KEYS)
        self.addCleanup(self.quotas_client.update_quota_set,
                        self.demo_tenant_id, **cleanup_quota_set)

    @decorators.idempotent_id('427c9f0c-982e-403d-ae45-c05f4d6322ff')
    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:quotas:show")
    def test_list_quotas(self):
        with self.rbac_utils.override_role(self):
            self.quotas_client.show_quota_set(self.demo_tenant_id)

    @decorators.idempotent_id('e47cf444-2753-4983-be6d-fc0d6523720f')
    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:quotas:show")
    def test_list_quotas_usage_true(self):
        with self.rbac_utils.override_role(self):
            self.quotas_client.show_quota_set(self.demo_tenant_id,
                                              params={'usage': True})

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:quotas:show")
    @decorators.idempotent_id('b3c7177e-b6b1-4d0f-810a-fc95606964dd')
    def test_list_default_quotas(self):
        with self.rbac_utils.override_role(self):
            self.quotas_client.show_default_quota_set(
                self.demo_tenant_id)

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:quotas:update")
    @decorators.idempotent_id('60f8f421-1630-4953-b449-b22af32265c7')
    def test_update_quota_set(self):
        self._restore_default_quota_set()
        new_quota_set = {'gigabytes': 1009,
                         'volumes': 11,
                         'snapshots': 11}
        # Update limits for all quota resources.
        with self.rbac_utils.override_role(self):
            self.quotas_client.update_quota_set(
                self.demo_tenant_id, **new_quota_set)

    @decorators.idempotent_id('329bdb88-5132-4810-b1fc-350d181577e3')
    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:quotas:delete")
    def test_delete_quota_set(self):
        self._restore_default_quota_set()

        with self.rbac_utils.override_role(self):
            self.quotas_client.delete_quota_set(self.demo_tenant_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.tests import base

from patrole_tempest_plugin.hacking import checks


class RBACHackingTestCase(base.TestCase):

    def test_import_no_clients_in_api_tests(self):
        for client in checks.PYTHON_CLIENTS:
            import_string = "import " + client + "client"
            self.assertTrue(checks.import_no_clients_in_api_tests(
                import_string,
                "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))
            self.assertFalse(checks.import_no_clients_in_api_tests(
                import_string,
                "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
            self.assertFalse(checks.import_no_clients_in_api_tests(
                import_string,
                "./patrole_tempest_plugin/tests/unit/fake_test.py"))
            self.assertFalse(checks.import_no_clients_in_api_tests(
                import_string,
                "./patrole_tempest_plugin/fake_test.py"))

    def test_no_setup_teardown_class_for_tests(self):
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertIsNone(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls): # noqa",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):",
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):",
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertIsNone(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls): # noqa",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):",
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):",
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))

    def test_no_vi_headers(self):
        self.assertTrue(checks.no_vi_headers(
            "# vim: tabstop=4", 1, range(250)))
        self.assertTrue(checks.no_vi_headers(
            "# vim: tabstop=4", 249, range(250)))

    def test_service_tags_not_in_module_path(self):
        self.assertTrue(checks.service_tags_not_in_module_path(
            "@utils.services('volume')",
            "./patrole_tempest_plugin/tests/api/volume/fake_test_rbac.py"))
        self.assertFalse(checks.service_tags_not_in_module_path(
            "@utils.services('image')",
            "./patrole_tempest_plugin/tests/api/volume/fake_test_rbac.py"))

    def test_no_hyphen_at_end_of_rand_name(self):
        self.assertIsNone(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test')",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertIsNone(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test')",
            "./patrole_tempest_plugin/tests/api/compute/fake_test_rbac.py"))
        self.assertIsNone(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test')",
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
        self.assertIsNone(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test')",
            "./patrole_tempest_plugin/tests/unit/fake_test.py"))
        self.assertTrue(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test-')",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertTrue(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test-')",
            "./patrole_tempest_plugin/tests/api/compute/fake_test_rbac.py"))
        self.assertTrue(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test-')",
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
        self.assertTrue(checks.no_hyphen_at_end_of_rand_name(
            "data_utils.rand_name('test-')",
            "./patrole_tempest_plugin/tests/unit/fake_test.py"))

    def test_no_mutable_default_args(self):
        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "  def test_function(test_param_1, test_param_2"))))
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "  def test_function(test_param_1, test_param_2={}"))))

    def test_no_testtools_skip_decorator(self):
        self.assertEqual(1, len(list(checks.no_testtools_skip_decorator(
            " @testtools.skip('Bug')"))))
        self.assertEqual(0, len(list(checks.no_testtools_skip_decorator(
            " @testtools.skipTest('reason')"))))
        self.assertEqual(0, len(list(checks.no_testtools_skip_decorator(
            " @testtools.skipUnless(reason, 'message')"))))
        self.assertEqual(0, len(list(checks.no_testtools_skip_decorator(
            " @testtools.skipIf(reason, 'message')"))))

    def test_use_rand_uuid_instead_of_uuid4(self):
        self.assertTrue(checks.use_rand_uuid_instead_of_uuid4(
            "new_uuid = uuid.uuid4()",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertTrue(checks.use_rand_uuid_instead_of_uuid4(
            "new_hex_uuid = uuid.uuid4().hex",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertIsNotNone(checks.use_rand_uuid_instead_of_uuid4(
            "new_uuid = data_utils.rand_uuid()",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertIsNotNone(checks.use_rand_uuid_instead_of_uuid4(
            "new_hex_uuid = data_utils.rand_uuid_hex()",
            "./patrole_tempest_plugin/tests/fake_test.py"))

    def _test_no_rbac_rule_validation_decorator(
            self, filename, with_other_decorators=True,
            with_rbac_decorator=True, expected_success=True):
        other_decorators = [
            "@decorators.idempotent_id(123)",
            "@decorators.attr(type=['slow'])",
            "@utils.requires_ext(extension='ext', service='svc')"
        ]

        if with_other_decorators:
            # Include multiple decorators to verify that this check works with
            # arbitrarily many decorators. These insert decorators above the
            # rbac_rule_validation decorator.
            for decorator in other_decorators:
                self.assertIsNone(checks.no_rbac_rule_validation_decorator(
                    " %s" % decorator, filename))
        if with_rbac_decorator:
            self.assertIsNone(checks.no_rbac_rule_validation_decorator(
                " @rbac_rule_validation.action('rule')",
                filename))
        if with_other_decorators:
            # Include multiple decorators to verify that this check works with
            # arbitrarily many decorators. These insert decorators between
            # the test and the @rbac_rule_validation decorator.
            for decorator in other_decorators:
                self.assertIsNone(checks.no_rbac_rule_validation_decorator(
                    " %s" % decorator, filename))
        final_result = checks.no_rbac_rule_validation_decorator(
            " def test_rbac_test",
            filename)
        if expected_success:
            self.assertIsNone(final_result)
        else:
            self.assertIsInstance(final_result, tuple)
            self.assertFalse(final_result[0])

    def test_no_rbac_rule_validation_decorator(self):
        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py")
        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py",
            False)
        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py",
            with_other_decorators=True, with_rbac_decorator=False,
            expected_success=False)
        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py",
            with_other_decorators=False, with_rbac_decorator=False,
            expected_success=False)

        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/scenario/fake_test.py")
        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/scenario/fake_test.py",
            False)
        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/scenario/fake_test.py",
            with_other_decorators=True, with_rbac_decorator=False,
            expected_success=False)
        self._test_no_rbac_rule_validation_decorator(
            "./patrole_tempest_plugin/tests/scenario/fake_test.py",
            with_other_decorators=False, with_rbac_decorator=False,
            expected_success=False)

    def test_no_rbac_suffix_in_test_filename(self):
        self.assertFalse(checks.no_rbac_suffix_in_test_filename(
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertFalse(checks.no_rbac_suffix_in_test_filename(
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
        self.assertFalse(checks.no_rbac_suffix_in_test_filename(
            "./patrole_tempest_plugin/tests/unit/fake_test.py"))
        self.assertFalse(checks.no_rbac_suffix_in_test_filename(
            "./patrole_tempest_plugin/tests/api/fake_rbac_base.py"))
        self.assertFalse(checks.no_rbac_suffix_in_test_filename(
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))
        self.assertTrue(checks.no_rbac_suffix_in_test_filename(
            "./patrole_tempest_plugin/tests/api/fake_test.py"))

    def test_no_rbac_test_suffix_in_test_class_name(self):
        self.assertFalse(checks.no_rbac_test_suffix_in_test_class_name(
            "class FakeTest",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertFalse(checks.no_rbac_test_suffix_in_test_class_name(
            "class FakeTest",
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
        self.assertFalse(checks.no_rbac_test_suffix_in_test_class_name(
            "class FakeTest",
            "./patrole_tempest_plugin/tests/unit/fake_test.py"))
        self.assertFalse(checks.no_rbac_test_suffix_in_test_class_name(
            "class FakeTest",
            "./patrole_tempest_plugin/tests/api/fake_rbac_base.py"))
        self.assertFalse(checks.no_rbac_test_suffix_in_test_class_name(
            "class FakeRbacTest",
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))
        self.assertTrue(checks.no_rbac_test_suffix_in_test_class_name(
            "class FakeTest",
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))

    def test_no_client_alias_in_test_cases(self):
        self.assertFalse(checks.no_client_alias_in_test_cases(
            "  self.client",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertFalse(checks.no_client_alias_in_test_cases(
            "  cls.client",
            "./patrole_tempest_plugin/tests/fake_test.py"))
        self.assertFalse(checks.no_client_alias_in_test_cases(
            "  self.client",
            "./patrole_tempest_plugin/tests/unit/fake_test.py"))
        self.assertFalse(checks.no_client_alias_in_test_cases(
            "  cls.client",
            "./patrole_tempest_plugin/tests/unit/fake_test.py"))
        self.assertFalse(checks.no_client_alias_in_test_cases(
            "  self.client",
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
        self.assertFalse(checks.no_client_alias_in_test_cases(
            "  cls.client",
            "./patrole_tempest_plugin/tests/scenario/fake_test.py"))
        self.assertTrue(checks.no_client_alias_in_test_cases(
            "  self.client",
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))
        self.assertTrue(checks.no_client_alias_in_test_cases(
            "  cls.client",
            "./patrole_tempest_plugin/tests/api/fake_test_rbac.py"))



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityProjectV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_project")
    @decorators.idempotent_id('0f148510-63bf-11e6-1564-080044d0d904')
    def test_create_project(self):
        with self.rbac_utils.override_role(self):
            self.setup_test_project()

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_project")
    @decorators.idempotent_id('0f148510-63bf-11e6-1564-080044d0d905')
    def test_update_project(self):
        project = self.setup_test_project()
        new_desc = data_utils.rand_name(
            self.__class__.__name__ + '-description')

        with self.rbac_utils.override_role(self):
            self.projects_client.update_project(project['id'],
                                                description=new_desc)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_project")
    @decorators.idempotent_id('0f148510-63bf-11e6-1564-080044d0d906')
    def test_delete_project(self):
        project = self.setup_test_project()

        with self.rbac_utils.override_role(self):
            self.projects_client.delete_project(project['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_project")
    @decorators.idempotent_id('0f148510-63bf-11e6-1564-080044d0d907')
    def test_show_project(self):
        project = self.setup_test_project()

        with self.rbac_utils.override_role(self):
            self.projects_client.show_project(project['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_projects")
    @decorators.idempotent_id('0f148510-63bf-11e6-1564-080044d0d908')
    def test_list_projects(self):
        with self.rbac_utils.override_role(self):
            self.projects_client.list_projects()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class VolumeQOSV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def setup_clients(cls):
        super(VolumeQOSV3RbacTest, cls).setup_clients()
        cls.qos_client = cls.os_primary.volume_qos_v2_client

    def _create_test_qos_specs(self, name=None, consumer=None, **kwargs):
        name = name or data_utils.rand_name(self.__class__.__name__ + '-QoS')
        consumer = consumer or 'front-end'
        qos_specs = self.qos_client.create_qos(
            name=name, consumer=consumer, **kwargs)['qos_specs']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.qos_client.delete_qos, qos_specs['id'])
        return qos_specs

    @rbac_rule_validation.action(
        service="cinder", rule="volume_extension:qos_specs_manage:create")
    @decorators.idempotent_id('4f9f45f0-b379-4577-a279-cec3e917cbec')
    def test_create_qos_with_consumer(self):
        with self.rbac_utils.override_role(self):
            self._create_test_qos_specs()

    @rbac_rule_validation.action(
        service="cinder", rule="volume_extension:qos_specs_manage:delete")
    @decorators.idempotent_id('fbc8a77e-6b6d-45ae-bebe-c496eb8f06f7')
    def test_delete_qos_with_consumer(self):
        qos = self._create_test_qos_specs()
        with self.rbac_utils.override_role(self):
            self.qos_client.delete_qos(qos['id'])

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:qos_specs_manage:get")
    @decorators.idempotent_id('22aff0dd-0343-408d-ae80-e77551956e14')
    def test_show_qos(self):
        qos = self._create_test_qos_specs()
        with self.rbac_utils.override_role(self):
            self.qos_client.show_qos(qos['id'])['qos_specs']

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:"
                                      "qos_specs_manage:get_all")
    @decorators.idempotent_id('ff1e98f3-d456-40a9-96d4-c7e4a55dcffa')
    def test_get_association_qos(self):
        qos = self._create_test_qos_specs()
        vol_type = self.create_volume_type()['id']
        self.qos_client.associate_qos(qos['id'], vol_type)
        self.addCleanup(self.qos_client.disassociate_qos, qos['id'], vol_type)

        with self.rbac_utils.override_role(self):
            self.qos_client.show_association_qos(qos['id'])

    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:qos_specs_manage:get_all")
    @decorators.idempotent_id('546b8bb1-04a4-4387-9506-a538a7f3cd6a')
    def test_list_qos(self):
        with self.rbac_utils.override_role(self):
            self.qos_client.list_qos()['qos_specs']

    @rbac_rule_validation.action(
        service="cinder", rule="volume_extension:qos_specs_manage:update")
    @decorators.idempotent_id('89b630b7-c170-47c3-ac80-50ed425c2d98')
    def test_set_qos_key(self):
        qos = self._create_test_qos_specs()
        with self.rbac_utils.override_role(self):
            self.qos_client.set_qos_key(
                qos['id'], iops_bytes='500')['qos_specs']

    @rbac_rule_validation.action(
        service="cinder", rule="volume_extension:qos_specs_manage:update")
    @decorators.idempotent_id('6c50c837-de77-4dae-a2ec-30e05c62969c')
    def test_unset_qos_key(self):
        qos = self._create_test_qos_specs()
        self.qos_client.set_qos_key(qos['id'], iops_bytes='500')['qos_specs']

        with self.rbac_utils.override_role(self):
            self.qos_client.unset_qos_key(qos['id'], ['iops_bytes'])
        waiters.wait_for_qos_operations(self.qos_client, qos['id'],
                                        'qos-key-unset', args=['iops_bytes'])

    @rbac_rule_validation.action(
        service="cinder", rule="volume_extension:qos_specs_manage:update")
    @decorators.idempotent_id('2047b347-8bbe-405e-bf5a-c75a0d7e3930')
    def test_associate_qos(self):
        qos = self._create_test_qos_specs()
        vol_type = self.create_volume_type()['id']

        with self.rbac_utils.override_role(self):
            self.qos_client.associate_qos(qos['id'], vol_type)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.qos_client.disassociate_qos, qos['id'], vol_type)

    @rbac_rule_validation.action(
        service="cinder", rule="volume_extension:qos_specs_manage:update")
    @decorators.idempotent_id('f12aeca1-0c02-4f33-b805-014171e5b2d4')
    def test_disassociate_qos(self):
        qos = self._create_test_qos_specs()
        vol_type = self.create_volume_type()['id']
        self.qos_client.associate_qos(qos['id'], vol_type)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.qos_client.disassociate_qos, qos['id'], vol_type)

        with self.rbac_utils.override_role(self):
            self.qos_client.disassociate_qos(qos['id'], vol_type)
        waiters.wait_for_qos_operations(self.qos_client, qos['id'],
                                        'disassociate', args=vol_type)

    @rbac_rule_validation.action(
        service="cinder", rule="volume_extension:qos_specs_manage:update")
    @decorators.idempotent_id('9f6e664d-a5d9-4e71-b122-73a3086be1b9')
    def test_disassociate_all_qos(self):
        qos = self._create_test_qos_specs()
        vol_type = self.create_volume_type()['id']
        self.qos_client.associate_qos(qos['id'], vol_type)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.qos_client.disassociate_qos, qos['id'], vol_type)

        with self.rbac_utils.override_role(self):
            self.qos_client.disassociate_all_qos(qos['id'])
        waiters.wait_for_qos_operations(self.qos_client, qos['id'],
                                        'disassociate-all')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class AgentsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(AgentsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-agents', 'compute'):
            raise cls.skipException(
                '%s skipped as os-agents not enabled' % cls.__name__)

    def _param_helper(self, **kwargs):
        rand_key = 'architecture'
        if rand_key in kwargs:
            # NOTE: The rand_name is for avoiding agent conflicts.
            # If you try to create an agent with the same hypervisor,
            # os and architecture as an existing agent, Nova will return
            # an HTTPConflict or HTTPServerError.
            kwargs[rand_key] = data_utils.rand_name(kwargs[rand_key])
        return kwargs

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-agents")
    @decorators.idempotent_id('d1bc6d97-07f5-4f45-ac29-1c619a6a7e27')
    def test_list_agents_rbac(self):
        with self.rbac_utils.override_role(self):
            self.agents_client.list_agents()

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-agents")
    @decorators.idempotent_id('77d6cae4-1ced-47f7-af2e-3d6a45958fd6')
    def test_create_agent(self):
        params = {'hypervisor': 'kvm', 'os': 'win', 'architecture': 'x86',
                  'version': '7.0', 'url': 'xxx://xxxx/xxx/xxx',
                  'md5hash': 'add6bb58e139be103324d04d82d8f545'}
        with self.rbac_utils.override_role(self):
            body = self.agents_client.create_agent(**params)['agent']
        self.addCleanup(self.agents_client.delete_agent,
                        body['agent_id'])

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-agents")
    @decorators.idempotent_id('b22f2681-9ffb-439b-b240-dae503e41020')
    def test_update_agent(self):
        params = self._param_helper(
            hypervisor='common', os='linux',
            architecture='x86_64', version='7.0',
            url='xxx://xxxx/xxx/xxx',
            md5hash='add6bb58e139be103324d04d82d8f545')
        body = self.agents_client.create_agent(**params)['agent']
        self.addCleanup(self.agents_client.delete_agent,
                        body['agent_id'])
        update_params = self._param_helper(
            version='8.0',
            url='xxx://xxxx/xxx/xxx2',
            md5hash='add6bb58e139be103324d04d82d8f547')

        with self.rbac_utils.override_role(self):
            self.agents_client.update_agent(body['agent_id'], **update_params)

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-agents")
    @decorators.idempotent_id('c5042af8-0682-43b0-abc4-bf33349e23dd')
    def test_delete_agent(self):
        params = self._param_helper(
            hypervisor='common', os='linux',
            architecture='x86_64', version='7.0',
            url='xxx://xxxx/xxx/xxx',
            md5hash='add6bb58e139be103324d04d82d8f545')
        body = self.agents_client.create_agent(**params)['agent']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.agents_client.delete_agent,
                        body['agent_id'])
        with self.rbac_utils.override_role(self):
            self.agents_client.delete_agent(body['agent_id'])



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class ServicesRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(ServicesRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-services', 'compute'):
            raise cls.skipException(
                '%s skipped as os-services not enabled' % cls.__name__)

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-services")
    @decorators.idempotent_id('7472261b-9c6d-453a-bcb3-aecaa29ad281')
    def test_list_services(self):
        with self.rbac_utils.override_role(self):
            self.services_client.list_services()['services']
